// NetClick.cpp
//
// A sample application that makes use of the
// FCommunicator library.  Start it from a Terminal
// window one of two different ways:
//
// To listen on a port:
//
//	NetClick -l 1234
//
// To connect to a listening NetClick:
//
//	NetClick -c host.domain.com 1234
//
// The window will turn grey once you've connected.
// When you click in the window, a dot will appear
// at that point on the other side.  Useless, but
// serves as a somewhat good example. :-) 
//

#include <stdio.h>

#include "NetClick.h"
#include "Communicator.h"

// main
//
// Standard cut-n-paste main() function.
//
int main(void)
{
	FApplication *app;
	
	app = new FApplication();
	app->Run();
	delete app;
	
	return(0);
}

// ----------------------------------------

// FApplication::FApplication
//
// Simple constructor -- create the window, the
// view, and show the window.
//
FApplication::FApplication()
			: BApplication('nclk')
{
	BRect r;
		
	// Create the window.
	//
	r.Set(100, 100, 300, 300);
	fWindow = new FWindow(r);

	// Offset it to the origin and make the view.
	//
	r.OffsetTo(B_ORIGIN);
	fView = new FView(r);

	// Attach the view to the window.
	//
	fWindow->AddChild(fView);

	// Show the window onscreen.
	//
	fWindow->Show();
}

// FAppilcation::ArgvReceived
//
// This handles the flags that people type on the
// command line.
//
void FApplication::ArgvReceived(int argc, char **argv)
{
	// Get a pointer to the FCommunicator that
	// the FView owns, and return if it's invalid.
	//
	FCommunicator *net = fView->Communicator();
	if (!net) { return; }
		
	// We need at least 3 arguments.
	//
	if (argc < 3) {
		printf("usage: NetClick [ -l port ] [ -c host port ]\n");
		Quit();
	}
	
	// Connect to a host -- we need at least 4.
	//
	if (!strcmp(argv[1], "-c")) {
		if (argc < 4) {
			printf("usage: NetClick [ -l port ] [ -c host port ]\n");
			Quit();
		}

		// We could do some sort of intelligent argument
		// validation here, but this is just an example,
		// so I'll make it a bad example! :-)
		//
		net->Connect(argv[2], atoi(argv[3]));
	}
	
	// Listen on a port.
	//
	else if (!strcmp(argv[1], "-l")) {
		net->Listen(atoi(argv[2]));
	}
	
	else {
		printf("usage: NetClick [ -l port ] [ -c host port ]\n");
	}
}

// ----------------------------------------

// FWindow::FWindow
//
// Nothing, really.  We derive from BWindow in
// order to override QuitRequested.
//
FWindow::FWindow(BRect r)
		: BWindow(r, "NetClick", B_TITLED_WINDOW,
				  B_NOT_RESIZABLE)
{
}

// FWindow::QuitRequested
//
// If they close the window, quit the application.
//
bool FWindow::QuitRequested(void)
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(TRUE);
}

// ----------------------------------------

// FView::FView
//
// The view sets up our FCommunicator object and
// an offscreen bitmap to draw into.
//
FView::FView(BRect r)
		: BView(r, "", B_FOLLOW_ALL, B_WILL_DRAW)
{
	// Instantiate a network object, so that
	// FApplication::ArgvReceived will have
	// something to tell when it wants us to
	// listen or connect.
	//
	fNetwork = new FCommunicator(this);

	// Set up an offscreen bitmap and a BView
	// to draw into it with.
	//
	fOffscreen = new BBitmap(r, B_COLOR_8_BIT, TRUE);
	fView = new BView(r, "", B_FOLLOW_ALL, B_WILL_DRAW);
	fOffscreen->AddChild(fView);
}

// FView::~FView
//
// Not much -- just make sure that fNetwork is
// picked up after.
//
FView::~FView()
{
	// Make sure our network stuff is cleaned up.
	//
	if (fNetwork) {
		fNetwork->Quit();
	}
}

// FView::Draw
//
// Just draw the offscreen bitmap into the
// specified BRect.
//
void FView::Draw(BRect r)
{
	DrawBitmap(fOffscreen, r, r);
}

// FView::MouseDown
// When the user clicks, send the coordinates
// of that point to the other side.
//
// FCommunicator::Send won't do anything if
// we're not connected yet, so it's safe to
// let them click to their heart's content
// until we are -- if we're not, this basically
// does nothing.
//
void FView::MouseDown(BPoint p)
{
	// Note that this doesn't do anything about
	// endian-ness, but since it only works from
	// BeOS to BeOS *IN THIS EXAMPLE*, it should
	// be okay not to worry about it.
	//
	fNetwork->Send((const char *)&p, sizeof(p));
}

// FView::MessageReceived
//
// All this does is handle network messages.
//
void FView::MessageReceived(BMessage *msg)
{
	BAlert *alert;
	
	switch(msg->what) {
	case F_NET_CONNECTED:
		// We're connected.  If we were listening,
		// that means someone connected to us.  If
		// we were connecting, then it means we've
		// connected to a listener.
		//
		// Turn the view grey to give some sort of
		// indication that we're connected.
		//
		fView->Window()->Lock();
		fView->SetHighColor(200, 200, 200);
		fView->FillRect(fView->Frame());
		fView->Sync();
		fView->Window()->Unlock();
		
		// Invalidate the view in order to make
		// sure the grey gets drawn.
		//
		Window()->Lock();
		Invalidate();
		Window()->Unlock();		
		break;
		
	case F_NET_RECEIVED:
		// We've received data from the network.
		//
		// First, extract that data from the message
		// that was sent.
		//
		long len;
		BPoint *p = (BPoint *)msg->FindData("data", B_RAW_TYPE, &len);			
		
		// Then, draw a dot at the coordinates that
		// were sent to us.
		//
		fView->Window()->Lock();
		rgb_color c = fView->HighColor();  // save the high color
		fView->SetHighColor(0, 0, 0);      // set it to black
		fView->FillEllipse(*p, 2.0, 2.0);  // draw a dot
		fView->SetHighColor(c);            // restore the high color
		fView->Sync();                     // sync the view
		fView->Window()->Unlock();

		// Figure out what part of the bitmap was
		// affected and invalidate that.
		//
		Window()->Lock();
		BRect ir;
		ir.Set((p->x - 2.0), (p->y - 2.0),
			   (p->x + 2.0), (p->y + 2.0));
		Invalidate(ir);
		Window()->Unlock();
		break;

	case F_NET_BYE:
		// The other side has disconnected.  Notify the
		// user and quit.
		//
		alert = new BAlert("", "Connection closed by foreign host.",
							   "OK");
		alert->Go();
		Window()->QuitRequested();
		break;

	case F_NET_FATAL:
		// Network SNAFU.  Tell the user and quit.
		//
		alert = new BAlert("", "There was a fatal network error.",
							   "OK", NULL, NULL, B_WIDTH_AS_USUAL,
							   B_STOP_ALERT);
		alert->Go();
		Window()->QuitRequested();
		break;
												
	default:
		// We didn't recognize the message; pass it along.
		//
		inherited::MessageReceived(msg);
		break;
	}
}
